import { api, API_CLIENT } from '../../../utils/api.ts';
import * as asserts from 'jsr:@std/assert';
import { EPHEMERAL_SERVER, get_ephemeral_listen_server, get_new_user, set_user_permissions } from '../../helpers.ts';
import { generateTotp } from '@stdext/crypto/totp';
import { clear_room_events_cache } from '../../../models/event.ts';

Deno.test({
	name: 'API - ROOMS - Delete',
	permissions: {
		env: true,
		read: true,
		write: true,
		net: true
	},
	fn: async () => {
		let test_server_info: EPHEMERAL_SERVER | null = null;
		try {
			test_server_info = await get_ephemeral_listen_server();
			const client: API_CLIENT = api({
				prefix: '/api',
				hostname: test_server_info.hostname,
				port: test_server_info.port
			});

			const user_info = await get_new_user(client);

			await set_user_permissions(client, user_info.user, user_info.session, [...user_info.user.permissions, 'rooms.create']);

			const new_room = await client.fetch('/rooms', {
				method: 'POST',
				headers: {
					'x-session_id': user_info.session.id,
					'x-totp': await generateTotp(user_info.session.secret)
				},
				json: {
					name: 'test delete room'
				}
			});

			asserts.assert(new_room);

			const deleted_room = await client.fetch(`/rooms/${new_room.id}`, {
				method: 'DELETE',
				headers: {
					'x-session_id': user_info.session.id,
					'x-totp': await generateTotp(user_info.session.secret)
				}
			});

			asserts.assert(deleted_room);
		} finally {
			clear_room_events_cache();
			if (test_server_info) {
				await test_server_info?.server?.stop();
			}
		}
	}
});
